package container;


import java.util.*;
import java.io.*;


class EccezioneDeposito extends Exception {
}

public class Deposito implements Serializable {

protected TreeMap<Integer, Container> deposito;

public Deposito () {
    deposito = new TreeMap<Integer, Container>();
}

public void ingresso(Container container, int posizione) throws EccezioneDeposito {
    if (deposito.containsKey(posizione))
      throw new EccezioneDeposito();
    deposito.put(posizione, container);
}

public Container uscita(int posizione) throws EccezioneDeposito {
    if (!deposito.containsKey(posizione))
      throw new EccezioneDeposito();
    return deposito.remove(posizione);
}

public Container ricerca(int codice) throws EccezioneDeposito {
    Container tmp;
        
    for (Iterator<Container> it = deposito.values().iterator(); it.hasNext();) {
       tmp = it.next();
       if (tmp.getCodice() == codice)
         return tmp.clone();
    }

    throw new EccezioneDeposito();
}

public float caricoTotale() {
    Container tmp;
    float carico = 0;

    for (Iterator<Container> it = deposito.values().iterator(); it.hasNext();) {
        tmp = it.next();
        carico += tmp.getCarico();
     }

     return carico;
}

public float caricoFrigo() {
    Container tmp;
    float carico = 0;

    for (Iterator<Container> it = deposito.values().iterator(); it.hasNext();) {
        tmp = it.next();
        if (tmp instanceof Container_frigo)
          carico += tmp.getCarico();
    }

    return carico;
}

public void salvaSuFile(String filename) throws IOException {
        FileOutputStream file = new FileOutputStream(filename);
        ObjectOutputStream output = new ObjectOutputStream(file);
        output.writeObject(deposito);
        file.close();
    }

public void caricaDaFile(String filename) throws IOException, ClassNotFoundException {
        FileInputStream file = new FileInputStream(filename);
        ObjectInputStream input = new ObjectInputStream(file);
        deposito = (TreeMap<Integer, Container>)input.readObject();
        file.close();
    }


public static void main (String args[]) throws IOException, ClassNotFoundException {
        Deposito deposito = new Deposito();
        Container c;
        
        Container c1 = new Container(123, 50);
        c1.setCarico(45);
        Container c2 = new Container(456, 25);
        c2.setCarico(20);
        Container_frigo c3 = new Container_frigo(789, 40);
        c3.setTemperatura(-20);
        c3.setCarico(40);
        
        try {
            deposito.ingresso(c1, 1);
            deposito.ingresso(c2, 10);
            deposito.ingresso(c3, 100);
        }
        catch (EccezioneDeposito e) {
            System.out.println("Eccezione ingresso");
        }
        
        System.out.println("Carico totale: "+deposito.caricoTotale());
        System.out.println("Carico refrigerato: "+deposito.caricoFrigo());
        
        try {
             c = deposito.ricerca(789);
             System.out.println(c);
        }
        catch (EccezioneDeposito e) {
            System.out.println("Eccezione ricerca");
        }
        
        deposito.salvaSuFile("deposito.bin");
        
        try {
             c = deposito.uscita(10);
             System.out.println(c);
        }
        catch (EccezioneDeposito e) {
            System.out.println("Eccezione uscita");
        }
        
        System.out.println("Carico totale: "+deposito.caricoTotale());
        System.out.println("Carico refrigerato: "+deposito.caricoFrigo());
        
        deposito.caricaDaFile("deposito.bin");
                
        System.out.println("Carico totale: "+deposito.caricoTotale());
        System.out.println("Carico refrigerato: "+deposito.caricoFrigo());
    }
}
